
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_RENDER
#define GR_SHQN_RENDER


#include "base.h"
#include "node.h"
#include "gcrwin32.h"
#include "viewer.h"
#include "status.h"
#include "pick.h"
#include "vertex.h"
#include "box.h"

	/** Window system intarface.
	    This class implements the main window system interface. The system
		framework must use a grRender object for scene rendering and
		user interaction.
	  */

class grRender
{
public:
		/** Default constructor.
		 */
	grRender();

		/** One-shot scene setting.
		    The user MAY use this function to setup the one-shot scene.
			The one-shot scene is renderend once during startup and contain
			all the general static settings.
		 */
	inline void SetInitScene( grNode * n );
		/** Rendering scene setting.
		    The user MUST use this function to setup the rendering scene.
			The rendering scene is rendered multiple time for visualization
			porpouse.
		 */
	inline void SetRunScene( grNode * n );

		/** Run-time rendering.
		    This callback must be used by the system framework to rendering the scene
			(i.e. by the OnDraw callback).
		 */
	void Render();

		/** Setup rendering.
		    This callback must be used by the system framework to rendering the
			inizialization scene, after the SetInitScene function. (i.e. on
			InitialUpdate callback).
		  */
	void Init();

		/** Current window size.
		    This function read the current window size.
			@param w width of the window.
			@param h height of the window
		  */
	void PortSize( int & w, int & h );

		/** Read the frame-buffer.
		    This function read the frame-buffer picture.
			@param format see opengl:glReadPixels for the list of formats
			@param type see opengl:glReadPixels for the list of types
			@param mem pointer of image memory (allocated by user!)
		  */
	void ReadPixel( GLenum format, GLenum type, GLvoid * mem );

		/** Read a part of the frame-buffer.
		    This function read the frame-buffer picture.
			@param x start of the sub section
			@param y start of the sub section
			@param w width of the sub section
			@param h height of the sub section
			@param format see opengl:glReadPixels for the list of formats
			@param type see opengl:glReadPixels for the list of types
			@param mem pointer of image memory (allocated by user!)
		  */
	void ReadSubPixel( int x, int y, int w, int h, GLenum format, GLenum type, GLvoid * mem );

		/** Convert a screen position to world position.
		    @param sx horizontal screen posizion
			@param sy vertical screen position
			@param p  world coordinates
			@return true if success
		  */
	bool ScreenToWorld( const int sx, const int sy, Coord3d & p );

		/** Dragger: start dragging.
		    The user must call this function to start the dragging.
			@param x horizontal mouse position
			@param y vertical mouse position
		 */
	void StartDrag( int x, int y );

		/** Dragger: dragging.
		    The user must call this function on moving the mouse during dragging.
			@param x horizontal mouse position
			@param y vertical mouse position
		 */
	void Drag( int x, int y );

		/** Dragger: end dragging.
		    The user must call this function to end the dragging.
			@param x horizontal mouse position
			@param y vertical mouse position
		 */
	void EndDrag( int x, int y );
	void GetDragRect( int * rect);

		/** Selection function.
		    The user may use this function to implement the selection of a set of nodes.
			@param x horizontal mouse position
			@param y vertical size position
			@param wx x selection range (in pixel)
			@param wy y selection range (in pixel)
			@param pr set of nodes selected
			@see grPickResult
		  */
	void PickNodes( int x, int y, double wx, double wy, grPickResult & pr );
		/** Selection function.
		    The user may use this function to implement the selection of a particular
			primitive of the spefied node.
			@param x horizontal mouse position
			@param y vertical size position
			@param wx x selection range (in pixel)
			@param wy y selection range (in pixel)
			@param pr pointer of a selected node
			@return the index of the selected node primive, -1 if none.
		  */
	unsigned int PickInNode( int x, int y, double wx, double wy, grNode * node );
		/** Selection function.
		    The user may use this function to implement the selection of a single node.
			@param x horizontal mouse position
			@param y vertical size position
			@param wx x selection range (in pixel)
			@param wy y selection range (in pixel)
			@param node the node selected
			@param prim index of the picked node primitive
			@return true if a node is selected, false otherwise
		  */
	bool Pick( int x, int y, double wx, double wy, grNode * & node, int & prim );

		/** Camera set tool.
		    This function set the current camera position for best viewing of the specified
			set of vertices.
		  */
	void grViewAll( const grVertex3f & vert);
		/** Camera set tool.
		    This function set the current camera position for best viewing of the specified
			bonding box.
		  */
	void grViewAll( const Boxd & boundbox );
		/** Camera set tool.
		    This function set the current camera position for best viewing of the specified
			bonding box.
		  */
	void grViewAll( const Boxf & boundbox );

		/** create callback.
			This function must be called by the system framework after the window creation
			(i.e. on OnCreate callback )
			@param hwnd the current window handle
		  */
	BOOL OnCreate( void * hwnd );
		/** destroy callback.
			This function must be called by the system framework on the window destruction
			(i.e. on OnDestroy callback )
			@param hwnd the current window handle
		  */
	void OnDestroy( void * hwnd );
		/** resize callback.
			This function must be called by the system framework on window size or resize
			(i.e. on OnSize callback ).
			@param hwnd the current window handle
			@param w new window width
			@param h new window height
		  */
	void OnSize( void * hwnd, int w, int h );

//private:
	GLbitfield  ClearMask;

	grNode *	init;		// Scena di inizializzazione
	grNode *	scene;		// Scena da renderizzare
	grWindExtra wextra;		// Dati extra sulla finestra
	grViewer	viewer;		// Virtual trackball
	grStatus    status;		// Stato di visualizzazione

private:
	static grRender * glOwner;				// "Propietario" del contesto gl
	grNode * FindByName( GLuint name );

	int dx,dy;		// Dragger data
	int ox,oy;
	GLint oldDrawBuffer;
	GLint oldDepthFunc;
	GLint oldLineStippleP;
	GLint oldLineStippleR;
	GLint curHeight;
	GLboolean BlendingIsE;
	GLint BlendFun1,BlendFun2;
	GLfloat oldEColor[4];

	inline void CheckOwner();
};


inline void grRender::CheckOwner()
{
	if(glOwner!=this)
	{
		wglMakeCurrent(wextra.ghDC, wextra.ghRC);
		glOwner = this;
	}
}


inline void grRender::SetRunScene( grNode * n )
{
	scene = n;
}


inline void grRender::SetInitScene( grNode * n )
{
	init = n;
}

#endif

